#include "Vector.h"
#include "CSRMatrix.h"
// конструктор по умолчанию
Vector::Vector() : vectorsize(0), vector(nullptr) {}

// запрещаем не¤вное преобразование
Vector::Vector(size_t vectorsize) : vectorsize(vectorsize), vector(new double[vectorsize])
{
	for (int i = 0; i < vectorsize; ++i)
	{
		vector[i] = 0;
	}
}

// конструктор
Vector::Vector(size_t vectorsize, double startingvalue) : vectorsize(vectorsize), vector(new double[vectorsize])// конструктор, который принимает размер вектора и начальное значение элементов
{
	for (int i = 0; i < vectorsize; ++i)
	{
		vector[i] = startingvalue;
	}
}

// конструктор копировани¤
Vector::Vector(const Vector& other) : vectorsize(other.vectorsize), vector(new double[other.vectorsize])
{
	for (int i = 0; i < other.vectorsize; ++i)
	{
		this->vector[i] = other.vector[i];
	}
}

// деструктор
Vector::~Vector()
{
	delete[]vector;
}

// перегрузка оператора = (ѕохож на конструктор копировани¤)
Vector& Vector::operator = (const Vector& other)
{
	this->vectorsize = other.vectorsize;
	if (this->vector != nullptr)
	{
		delete[] this->vector;
	}
	this->vector = new double[other.vectorsize];
	for (int i = 0; i < other.vectorsize; ++i)
	{
		this->vector[i] = other.vector[i];
	}
	return *this;
}

// префиксна¤ перегрузка инкремента
Vector& Vector::operator ++ ()
{
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] += 1; // или this->vector[i]++;
	}
	return *this;
}

// постфиксна¤ перегрузка инкремента
Vector Vector::operator ++ (int value)
{
	Vector temp(*this); // вызоветс¤ конструктор копировани¤
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] += 1; // или this->vector[i]++;
	}
	return temp;
}

// префиксна¤ перегрузка декремента
Vector& Vector::operator -- ()
{
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] -= 1;
	}
	return *this;
}

// постфиксна¤ перегрузка декремента
Vector Vector:: operator --(int value)
{
	Vector temp(*this);
	for (int i = 0; i < vectorsize; ++i)
	{
		this->vector[i] -= 1;
	}
	return temp;
}

// перегрузка операторов [] 
double& Vector::operator[](size_t index)
{
	return vector[index];
}
const double& Vector::operator [](size_t index) const
{
	return vector[index];
}

// перегрузка оператора сложени¤ двух векторов
Vector Vector::operator + (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		Vector temp(this->vectorsize, 0);
		for (int i = 0; i < this->vectorsize; ++i)
		{
			temp.vector[i] = this->vector[i] + other.vector[i];
		}
		return temp;
	}
}

// перегрузка оператора разницы двух векторов
Vector Vector::operator - (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		Vector temp(this->vectorsize, 0);
		for (int i = 0; i < this->vectorsize; ++i)
		{
			temp.vector[i] = this->vector[i] - other.vector[i];
		}
		return temp;
	}
}

// перегрузка оператора += дл¤ вектора
Vector& Vector::operator += (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			this->vector[i] += other.vector[i];
		}
		return *this;
	}
}

// перегрузка оператора -= дл¤ вектора
Vector& Vector::operator -= (const Vector& other)
{
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			this->vector[i] -= other.vector[i];
		}
		return *this;
	}
}

// перегрузка оператора += дл¤ double
Vector& Vector::operator += (double other)
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] += other;
	}
	return *this;
}

// перегрузка оператора -= дл¤ double
Vector& Vector::operator -= (double other)
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] -= other;
	}
	return *this;
}

// перегрузка оператора умножени¤ вектора на число
Vector Vector::operator *(double value) const
{
	for (int i = 0; i < this->vectorsize; ++i)
	{
		this->vector[i] *= value;
	}
	return *this;
}

// перегрузка оператора скал¤рного произведени¤ векторов
double Vector::operator *(const Vector& other)
{
	double multiplication = 0;
	if (this->vectorsize != other.vectorsize)
	{
		throw OutOfRangeException();
	}
	else
	{
		for (int i = 0; i < this->vectorsize; ++i)
		{
			multiplication += this->vector[i] * other.vector[i];
		}
		return multiplication;
	}
}

// перегрузка оператора приведени¤
Vector ::operator double* () const
{
	return vector;
}

// метод дл¤ получени¤ размерности вектора
size_t Vector::getsize() const
{
	return vectorsize;
}

// метод дл¤ вычислени¤ длины вектора
double Vector::VectorLength() const
{
	double sum = 0;
	for (int i = 0; i < vectorsize; ++i)
	{
		sum += vector[i] * vector[i];
	}
	return sqrt(sum);
}

// перегрузка оператора умножени¤ числа на вектор
Vector operator *(double value, const Vector& vector)
{
	Vector temp(vector);
	for (int i = 0; i < temp.vectorsize; ++i)
	{
		temp.vector[i] = value * temp.vector[i];
	}
	return temp;
}

// перегрузка оператора вывода из потока
std::ostream& operator <<(std::ostream& out, const Vector& vector)
{
	for (size_t i = 0; i < vector.vectorsize; ++i)
	{
		out << vector[i] << " ";
	}
	return out;// возвращаем поток
}

// перегрузка оператора ввода в поток
std::istream& operator >> (std::istream& in, Vector& vector)
{
	for (size_t i = 0; i < vector.getsize(); ++i)
	{
		in >> vector[i];
	}
	return in;
}

// перегрузка оператора умножени¤ вектора на матрицу и матрицы на вектор описаны в классе матриц
